<?php
/**
 * CTabView class file. 
 *
 * @author Qiang Xue <qiang.xue@gmail.com> extended by Pablo Cianci <freakpol@gmail.com> 
 * @link http://www.yiiframework.com/
 * @copyright Copyright &copy; 2008-2009 Yii Software LLC
 * @license http://www.yiiframework.com/license/
 */

/**
 * EActiveTabView displays contents in multiple tabs, storing the actual selected tab inside a hidden field, so leter we can ask fot this field.
 *
 * At any time, only one tab is visible. Users can click on the tab header
 * to switch to see another tab of content.
 *
 * JavaScript is used to control the tab switching. If JavaScript is disabled,
 * EActiveTabView still manages to display the content in a semantically appropriate way.
 * but no way to store the hidden field with the actual selected tab.
 *
 * To specify contents and their tab structure, configure the {@link tabs} property.
 * The {@link tabs} property takes an array with tab ID being mapped tab definition.
 * Each tab definition is an array of the following structure:
 * <ul>
 * <li>title: the tab title.</li>
 * <li>content: the content to be displayed in the tab.</li>
 * <li>view: the name of the view to be displayed in this tab.
 * The view will be rendered using the current controller's
 * {@link CController::renderPartial} method.
 * When both 'content' and 'view' are specified, 'content' will take precedence.
 * </li>
 * <li>url: a URL that the user browser will be redirected to when clicking on this tab.</li>
 * </ul>
 *
 * For example, the {@link tabs} property can be configured as follows,
 * <pre>
 * array(
 *     'tab1'=>array(
 *           'title'=>'tab 1 title',
 *           'view'=>'view1',
 *     ),
 *     'tab2'=>array(
 *           'title'=>'tab 2 title',
 *           'url'=>'http://www.yiiframework.com/',
 *     ),
 * )
 * </pre>
 *
 * You must define the name of the hidden fiel through 'name', or yo can define a model and an atribute
 * through 'model' and 'attribure'
 *
 * For example
 * <pre> 
 *	$this->widget('application.extensions.ActiveTabView.EActiveTabView',
 *		array('tabs'=>$Tabs, 'viewData'=>$viewData,
 *		'model'=>$someModel,'attribute'=>'attributeOfThatModel'
 * ));
 * </pre>
 * or 
 * <pre> 
 *	$this->widget('application.extensions.ActiveTabView.EActiveTabView',
 *		array('tabs'=>$Tabs, 'viewData'=>$viewData,
 *		'name'=>'hiddenFieldName'
 * ));
 * </pre>
 *
 *
 * By default, the first tab will be activated. To activate a different tab
 * when the page is initially loaded, set {@link activeTab} to be the ID of the desired tab.
 *
 * @author Qiang Xue <qiang.xue@gmail.com>
 * @version $Id: CTabView.php 745 2009-02-25 21:45:42Z qiang.xue $
 * @package system.web.widgets
 * @since 1.0
 */
class EActiveTabView extends CInputWidget
{
	/**
	 * Default CSS class for the tab container
	 */
	const CSS_CLASS='yiiTab';

	/**
	 * @var mixed the CSS file used for the widget. Defaults to null, meaning
	 * using the default CSS file included together with the widget.
	 * If false, no CSS file will be used. Otherwise, the specified CSS file
	 * will be included when using this widget.
	 */
	public $cssFile;
	/**
	 * @var string the ID of the tab that should be activated when the page is initially loaded.
	 * If not set, the first tab will be activated.
	 */
	public $activeTab;
	/**
	 * @var array the data that will be passed to the partial view rendered by each tab.
	 */
	public $viewData;
	/**
	 * @var array additional HTML options to be rendered in the container tag.
	 */
	public $htmlOptions;
	/**
	 * @var array tab definitions. The array keys are the IDs,
	 * and the array values are the corresponding tab contents.
	 * Each array value must be an array with the following elements:
	 * <ul>
	 * <li>title: the tab title. You need to make sure this is HTML-encoded.</li>
	 * <li>content: the content to be displayed in the tab.</li>
	 * <li>view: the name of the view to be displayed in this tab.
	 * The view will be rendered using the current controller's
	 * {@link CController::renderPartial} method.
	 * When both 'content' and 'view' are specified, 'content' will take precedence.
	 * </li>
	 * <li>url: a URL that the user browser will be redirected to when clicking on this tab.</li>
	 * </ul>
	 * <pre>
	 * array(
	 *     'tab1'=>array(
	 *           'title'=>'tab 1 title',
	 *           'view'=>'view1',
	 *     ),
	 *     'tab2'=>array(
	 *           'title'=>'tab 2 title',
	 *           'url'=>'http://www.yiiframework.com/',
	 *     ),
	 * )
	 * </pre>
	 */
	public $tabs=array();
	
	public $baseUrl = '';

	//public $hiddenName = 'activeTabView';
	
	/*
	 * Publishes the assets
	 */
	
	public function publishAssets()
   {
      $dir = dirname(__FILE__).DIRECTORY_SEPARATOR.'js';
      $this->baseUrl = Yii::app()->getAssetManager()->publish($dir);
   }	

	/**
	 * Runs the widget.
	 */
	public function run()
	{
		$this->publishAssets();	
	
		if(empty($this->tabs))
			return;

		if($this->activeTab===null || !isset($this->tabs[$this->activeTab]))
		{
			foreach($this->tabs as $id=>$tab)
			{
				$this->activeTab=$id;
				break;
			}
		}

		$htmlOptions=$this->htmlOptions;
		$htmlOptions['id']=$this->getId();
		if(!isset($htmlOptions['class']))
			$htmlOptions['class']=self::CSS_CLASS;

		$this->registerClientScript();

		echo CHtml::openTag('div',$htmlOptions)."\n";
		$this->renderHeader();
		$this->renderBody();
		echo CHtml::closeTag('div');
	}

	protected function resolveNameID()
	{
		if($this->name!==null)
			$name=$this->name;
		else if($this->hasModel())
			$name=get_class($this->model).'['.$this->attribute.']';
		else if(isset($this->htmlOptions['name']))
			$name=$this->htmlOptions['name'];
		else
			throw new CException(Yii::t('yii','{class} must specify "model" and "attribute" or "name" property values.',array('{class}'=>get_class($this))));

		/*if(($id=$this->getId(false))===null)
		{*/
		/*	if(isset($this->htmlOptions['id']))
				$id=$this->htmlOptions['id'];
			else*/
				$id=CHtml::getIdByName($name);
		//}

		return array($name,$id);
	}




	/**
	 * Registers the needed CSS and JavaScript.
	 */
	public function registerClientScript()
	{
	
		if ($this->baseUrl === '')
         throw new CException(Yii::t('EActiveTabView', 'baseUrl must be set. This is done automatically by calling publishAssets()'));

		$cs=Yii::app()->getClientScript();
		//$cs->registerCoreScript('application.extensions.ActiveTabView.jquery.yiiactivetab.js');
		
		$cs->registerCoreScript('yiiactivetab');		
		$id = $this->getId();
		
		list($name, $fieldId) = $this->resolveNameId();
		$cs->registerScriptFile($this->baseUrl.'/activetabview.js');
		
		//$cs->registerScript('Yii.CTabView#'.$id,"jQuery(\"#{$id}\").yiitab();");
		$cs->registerScript('EActiveTabView#'.$id,
									"$(\"#{$id}\").activetabview(\"#{$fieldId}\");");

		if($this->cssFile!==false)
			self::registerCssFile($this->cssFile);
	}

	/**
	 * Registers the needed CSS file.
	 * @param string the CSS URL. If null, a default CSS URL will be used.
	 * @since 1.0.2
	 */
	public static function registerCssFile($url=null)
	{
		$cs=Yii::app()->getClientScript();
		if($url===null)
			$url=$cs->getCoreScriptUrl().'/yiitab/jquery.yiitab.css';
		$cs->registerCssFile($url,'screen');
	}

	/**
	 * Renders the header part.
	 */
	protected function renderHeader()
	{
		list($name, $myid) = $this->resolveNameId();
		
		echo "<input type=\"hidden\" value=\"".$this->activeTab."\" name=\"".$name."\" id=\"".$myid."\" />";
		echo "<ul class=\"tabs\">\n";
		foreach($this->tabs as $id=>$tab)
		{
			$title=isset($tab['title'])?$tab['title']:'undefined';
			$active=$id===$this->activeTab?' class="active"' : '';
			$url=isset($tab['url'])?$tab['url']:"#{$id}";
			echo "<li><a href=\"{$url}\"{$active}>{$title}</a></li>\n";
		}
		echo "</ul>\n";
	}

	/**
	 * Renders the body part.
	 */
	protected function renderBody()
	{
		foreach($this->tabs as $id=>$tab)
		{
			$inactive=$id!==$this->activeTab?' style="display:none"' : '';
			echo "<div class=\"view\" id=\"{$id}\"{$inactive}>\n";
			if(isset($tab['content']))
				echo $tab['content'];
			else if(isset($tab['view']))
				$vd = $this->viewData;
				if (isset($tab['viewData']))
				{
					$vd = $tab['viewData'];
				}
			
				$this->getController()->renderPartial($tab['view'],$vd);
			echo "</div><!-- {$id} -->\n";
		}
	}
}
